#include "Parser.h"

#include <iostream>
using namespace std;

const Parser::token_info_t Parser::token_info_map[] =
{
	{ TOKEN_LABEL,                      "LABEL",                      TOKEN_META_PSEUDO },
	{ TOKEN_EXPRESSION,                 "EXPRESSION",                 TOKEN_META_PSEUDO },
	{ TOKEN_DIRECT_DECLARATOR,          "DIRECT DECLARATOR",          TOKEN_META_PSEUDO },
	{ TOKEN_TYPE_SPECIFIER,             "TYPE SPECIFIER",             TOKEN_META_PSEUDO },
	{ TOKEN_DECLARATION_SPECIFIERS,     "DECLARATION SPECIFIERS",     TOKEN_META_PSEUDO },
	{ TOKEN_PARAMETER_DECLARATION,      "PARAMETER DECLARATION",      TOKEN_META_PSEUDO },
	{ TOKEN_PARAMETER_LIST,             "PARAMETER LIST",             TOKEN_META_PSEUDO },
	{ TOKEN_PARAMETER_TYPE_LIST,        "PARAMETER TYPE LIST",        TOKEN_META_PSEUDO },
	{ TOKEN_TRANSLATION_UNIT,           "TRANSLATION UNIT",           TOKEN_META_PSEUDO },
	{ TOKEN_EXTERNAL_DECLARATION,       "EXTERNAL DECLARATION",       TOKEN_META_PSEUDO },
	{ TOKEN_INIT_DECLARATOR_LIST,       "INIT DECLARATOR LIST",       TOKEN_META_PSEUDO },
	{ TOKEN_DECLARATOR,                 "DECLARATOR",                 TOKEN_META_PSEUDO },
	{ TOKEN_DECLARATION,                "DECLARATION",                TOKEN_META_PSEUDO },
	{ TOKEN_INIT_DECLARATOR,            "INIT DECLARATOR",            TOKEN_META_PSEUDO },
	{ TOKEN_BLOCK_ITEM_LIST,            "BLOCK ITEM LIST",            TOKEN_META_PSEUDO },
	{ TOKEN_BLOCK_ITEM,                 "BLOCK ITEM",                 TOKEN_META_PSEUDO },
	{ TOKEN_FUNCTION_DEFINITION,        "FUNCTION DEFINITION",        TOKEN_META_PSEUDO },
	{ TOKEN_PRIMARY_EXPRESSION,         "PRIMARY EXPRESSION",         TOKEN_META_PSEUDO },
	{ TOKEN_ARGUMENT_EXPRESSION_LIST,   "ARGUMENT EXPRESSION LIST",   TOKEN_META_PSEUDO },
	{ TOKEN_ASSIGNMENT_EXPRESSION,      "ASSIGNMENT EXPRESSION",      TOKEN_META_PSEUDO },
	{ TOKEN_POSTFIX_EXPRESSION,         "POSTFIX EXPRESSION",         TOKEN_META_PSEUDO },
	{ TOKEN_UNARY_EXPRESSION,           "UNARY EXPRESSION",           TOKEN_META_PSEUDO },
	{ TOKEN_CAST_EXPRESSION,            "CAST EXPRESSION",            TOKEN_META_PSEUDO },
	{ TOKEN_MULTIPLICATIVE_EXPRESSION,  "MULTIPLICATIVE EXPRESSION",  TOKEN_META_PSEUDO },
	{ TOKEN_ADDITIVE_EXPRESSION,        "ADDITIVE EXPRESSION",        TOKEN_META_PSEUDO },
	{ TOKEN_SHIFT_EXPRESSION,           "SHIFT EXPRESSION",           TOKEN_META_PSEUDO },
	{ TOKEN_RELATIONAL_EXPRESSION,      "RELATIONAL EXPRESSION",      TOKEN_META_PSEUDO },
	{ TOKEN_AND_EXPRESSION,             "AND EXPRESSION",             TOKEN_META_PSEUDO },
	{ TOKEN_EXCLUSIVE_OR_EXPRESSION,    "EXCLUSIVE OR EXPRESSION",    TOKEN_META_PSEUDO },
	{ TOKEN_INCLUSIVE_OR_EXPRESSION,    "INCLUSIVE OR EXPRESSION",    TOKEN_META_PSEUDO },
	{ TOKEN_LOGICAL_AND_EXPRESSION,     "LOGICAL AND EXPRESSION",     TOKEN_META_PSEUDO },
	{ TOKEN_LOGICAL_OR_EXPRESSION,      "LOGICAL OR EXPRESSION",      TOKEN_META_PSEUDO },
	{ TOKEN_CONDITIONAL_EXPRESSION,     "CONDITIONAL EXPRESSION",     TOKEN_META_PSEUDO },
	{ TOKEN_EXPRESSION_STATEMENT,       "EXPRESSION STATEMENT",       TOKEN_META_PSEUDO },
	{ TOKEN_COMPOUND_STATEMENT,         "COMPOUND STATEMENT",         TOKEN_META_PSEUDO },
	{ TOKEN_SELECTION_STATEMENT,        "SELECTION STATEMENT",        TOKEN_META_PSEUDO },
	{ TOKEN_LABELED_STATEMENT,          "LABELED STATEMENT",          TOKEN_META_PSEUDO },
	{ TOKEN_JUMP_STATEMENT,             "JUMP STATEMENT",             TOKEN_META_PSEUDO },
	{ TOKEN_ITERATION_STATEMENT,        "ITERATION STATEMENT",        TOKEN_META_PSEUDO },
	{ TOKEN_POINTER,                    "POINTER",                    TOKEN_META_PSEUDO },
	{ TOKEN_CONSTANT,                   "CONSTANT",                   TOKEN_META_PSEUDO },
	{ TOKEN_SPECIFIER_QUALIFIER_LIST,   "SPECIFIER QUALIFIER LIST",   TOKEN_META_PSEUDO },
	{ TOKEN_ENUM_SPECIFIER,             "ENUM SPECIFIER",             TOKEN_META_PSEUDO },
	{ TOKEN_FUNCTION_SPECIFIER,         "FUNCTION SPECIFIER",         TOKEN_META_PSEUDO },
	{ TOKEN_TYPE_QUALIFIER,             "TYPE QUALIFIER",             TOKEN_META_PSEUDO },
	{ TOKEN_TYPE_QUALIFIER_LIST,        "TYPE QUALIFIER LIST",        TOKEN_META_PSEUDO },
	{ TOKEN_INITIALIZER_LIST,           "INITIALIZER LIST",           TOKEN_META_PSEUDO },
	{ TOKEN_STRUCT_DECLARATION,         "STRUCT DECLARATION",         TOKEN_META_PSEUDO },
	{ TOKEN_STRUCT_DECLARATION_LIST,    "STRUCT DECLARATION LIST",    TOKEN_META_PSEUDO },
	{ TOKEN_STRUCT_DECLARATOR,          "STRUCT DECLARATOR",          TOKEN_META_PSEUDO },
	{ TOKEN_STRUCT_DECLARATOR_LIST,     "STRUCT DECLARATOR LIST",     TOKEN_META_PSEUDO },
	{ TOKEN_PARAMETER,                  "PARAMETER",                  TOKEN_META_PSEUDO },
	{ TOKEN_INITIALIZER,                "INITIALIZER",                TOKEN_META_PSEUDO },
	{ TOKEN_STORAGE_CLASS_SPECIFIER,    "STORAGE CLASS SPECIFIER",    TOKEN_META_PSEUDO },
	{ TOKEN_STRUCT_OR_UNION_SPECIFIER,  "STRUCT OR UNION SPECIFIER",  TOKEN_META_PSEUDO },
	{ TOKEN_STATEMENT,                  "STATEMENT",                  TOKEN_META_PSEUDO },
	{ TOKEN_DESIGNATOR_LIST,            "DESIGNATOR LIST",            TOKEN_META_PSEUDO },
	{ TOKEN_IDENTIFIER_LIST,            "IDENTIFIER LIST",            TOKEN_META_PSEUDO },
	{ TOKEN_ENUMERATOR,                 "ENUMERATOR",                 TOKEN_META_PSEUDO },
	{ TOKEN_DECLARATION_LIST,           "DECLARATION LIST",           TOKEN_META_PSEUDO },
	{ TOKEN_ENUMERATOR_LIST,            "ENUMERATOR LIST",            TOKEN_META_PSEUDO },
	{ TOKEN_DIRECT_ABSTRACT_DECLARATOR, "DIRECT ABSTRACT DECLARATOR", TOKEN_META_PSEUDO },
	{ TOKEN_DESIGNATION,                "DESIGNATION",                TOKEN_META_PSEUDO },
	{ TOKEN_DESIGNATOR,                 "DESIGNATOR",                 TOKEN_META_PSEUDO },
	{ TOKEN_CONSTANT_EXPRESSION,        "CONSTANT EXPRESSION",        TOKEN_META_PSEUDO },
	{ TOKEN_AUTO,                       "AUTO",                       TOKEN_META_IDENTIFIER },
	{ TOKEN_BREAK,                      "BREAK",                      TOKEN_META_IDENTIFIER },
	{ TOKEN_CASE,                       "CASE",                       TOKEN_META_IDENTIFIER },
	{ TOKEN_CHAR,                       "CHAR",                       TOKEN_META_IDENTIFIER },
	{ TOKEN_CONST,                      "CONST",                      TOKEN_META_IDENTIFIER },
	{ TOKEN_CONTINUE,                   "CONTINUE",                   TOKEN_META_IDENTIFIER },
	{ TOKEN_DEFAULT,                    "DEFAULT",                    TOKEN_META_IDENTIFIER },
	{ TOKEN_DO,                         "DO",                         TOKEN_META_IDENTIFIER },
	{ TOKEN_DOUBLE,                     "DOUBLE",                     TOKEN_META_IDENTIFIER },
	{ TOKEN_ELSE,                       "ELSE",                       TOKEN_META_IDENTIFIER },
	{ TOKEN_ENUM,                       "ENUM",                       TOKEN_META_IDENTIFIER },
	{ TOKEN_EXTERN,                     "EXTERN",                     TOKEN_META_IDENTIFIER },
	{ TOKEN_FLOAT,                      "FLOAT",                      TOKEN_META_IDENTIFIER },
	{ TOKEN_FOR,                        "FOR",                        TOKEN_META_IDENTIFIER },
	{ TOKEN_GOTO,                       "GOTO",                       TOKEN_META_IDENTIFIER },
	{ TOKEN_IF,                         "IF",                         TOKEN_META_IDENTIFIER },
	{ TOKEN_INLINE,                     "INLINE",                     TOKEN_META_IDENTIFIER },
	{ TOKEN_INT,                        "INT",                        TOKEN_META_IDENTIFIER },
	{ TOKEN_LONG,                       "LONG",                       TOKEN_META_IDENTIFIER },
	{ TOKEN_REGISTER,                   "REGISTER",                   TOKEN_META_IDENTIFIER },
	{ TOKEN_RESTRICT,                   "RESTRICT",                   TOKEN_META_IDENTIFIER },
	{ TOKEN_RETURN,                     "RETURN",                     TOKEN_META_IDENTIFIER },
	{ TOKEN_SHORT,                      "SHORT",                      TOKEN_META_IDENTIFIER },
	{ TOKEN_SIGNED,                     "SIGNED",                     TOKEN_META_IDENTIFIER },
	{ TOKEN_SIZEOF,                     "SIZEOF",                     TOKEN_META_IDENTIFIER },
	{ TOKEN_STATIC,                     "STATIC",                     TOKEN_META_IDENTIFIER },
	{ TOKEN_STRUCT,                     "STRUCT",                     TOKEN_META_IDENTIFIER },
	{ TOKEN_SWITCH,                     "SWITCH",                     TOKEN_META_IDENTIFIER },
	{ TOKEN_TYPEDEF,                    "TYPEDEF",                    TOKEN_META_IDENTIFIER },
	{ TOKEN_UNION,                      "UNION",                      TOKEN_META_IDENTIFIER },
	{ TOKEN_UNSIGNED,                   "UNSIGNED",                   TOKEN_META_IDENTIFIER },
	{ TOKEN_VOID,                       "VOID",                       TOKEN_META_IDENTIFIER },
	{ TOKEN_VOLATILE,                   "VOLATILE",                   TOKEN_META_IDENTIFIER },
	{ TOKEN_WHILE,                      "WHILE",                      TOKEN_META_IDENTIFIER },
	{ TOKEN__BOOL,                      "_BOOL",                      TOKEN_META_IDENTIFIER },
	{ TOKEN__COMPLEX,                   "_COMPLEX",                   TOKEN_META_IDENTIFIER },
	{ TOKEN__IMAGINARY,                 "_IMAGINARY",                 TOKEN_META_IDENTIFIER },
	{ TOKEN_ELLIPSIS,                   "ELLIPSIS",                   TOKEN_META_IDENTIFIER },
	{ TOKEN_DART,                       "DART",                       TOKEN_META_IDENTIFIER },
	{ TOKEN_LS_ASSIGN,                  "LS_ASSIGN",                  TOKEN_META_IDENTIFIER },
	{ TOKEN_RS_ASSIGN,                  "RS_ASSIGN",                  TOKEN_META_IDENTIFIER },
	{ TOKEN_MUL_ASSIGN,                 "MUL_ASSIGN",                 TOKEN_META_IDENTIFIER },
	{ TOKEN_DIV_ASSIGN,                 "DIV_ASSIGN",                 TOKEN_META_IDENTIFIER },
	{ TOKEN_ADD_ASSIGN,                 "ADD_ASSIGN",                 TOKEN_META_IDENTIFIER },
	{ TOKEN_SUB_ASSIGN,                 "SUB_ASSIGN",                 TOKEN_META_IDENTIFIER },
	{ TOKEN_NEG_ASSIGN,                 "NEG_ASSIGN",                 TOKEN_META_IDENTIFIER },
	{ TOKEN_XOR_ASSIGN,                 "XOR_ASSIGN",                 TOKEN_META_IDENTIFIER },
	{ TOKEN_AND_ASSIGN,                 "AND_ASSIGN",                 TOKEN_META_IDENTIFIER },
	{ TOKEN_OR_ASSIGN,                  "OR_ASSIGN",                  TOKEN_META_IDENTIFIER },
	{ TOKEN_MOD_ASSIGN,                 "MOD_ASSIGN",                 TOKEN_META_IDENTIFIER },
	{ TOKEN_EQ_COMPARE,                 "EQ_COMPARE",                 TOKEN_META_IDENTIFIER },
	{ TOKEN_NE_COMPARE,                 "NE_COMPARE",                 TOKEN_META_IDENTIFIER },
	{ TOKEN_LE_COMPARE,                 "LE_COMPARE",                 TOKEN_META_IDENTIFIER },
	{ TOKEN_GE_COMPARE,                 "GE_COMPARE",                 TOKEN_META_IDENTIFIER },
	{ TOKEN_INCREMENT,                  "INCREMENT",                  TOKEN_META_IDENTIFIER },
	{ TOKEN_DECREMENT,                  "DECREMENT",                  TOKEN_META_IDENTIFIER },
	{ TOKEN_LOGICAL_AND,                "LOGICAL_AND",                TOKEN_META_IDENTIFIER },
	{ TOKEN_LOGICAL_OR,                 "LOGICAL_OR",                 TOKEN_META_IDENTIFIER },
	{ TOKEN_LS,                         "LS",                         TOKEN_META_IDENTIFIER },
	{ TOKEN_RS,                         "RS",                         TOKEN_META_IDENTIFIER },
	{ TOKEN_IDENTIFIER,                 "IDENTIFIER",                 TOKEN_META_IDENTIFIER },
	{ TOKEN_TYPE_NAME,                  "TYPE_NAME",                  TOKEN_META_IDENTIFIER },
	{ TOKEN_DECIMAL_CONSTANT,           "DECIMAL_CONSTANT",           TOKEN_META_IDENTIFIER },
	{ TOKEN_HEXADECIMAL_CONSTANT,       "HEXADECIMAL_CONSTANT",       TOKEN_META_IDENTIFIER },
	{ TOKEN_OCTAL_CONSTANT,             "OCTAL_CONSTANT",             TOKEN_META_IDENTIFIER },
	{ TOKEN_UNIVERSAL_CHARACTER_NAME,   "UNIVERSAL_CHARACTER_NAME",   TOKEN_META_IDENTIFIER },
	{ TOKEN_DECIMAL_FLOAT_CONSTANT,     "DECIMAL_FLOAT_CONSTANT",     TOKEN_META_IDENTIFIER },
	{ TOKEN_HEXADECIMAL_FLOAT_CONSTANT, "HEXADECIMAL_FLOAT_CONSTANT", TOKEN_META_IDENTIFIER },
	{ TOKEN_STRING,                     "STRING",                     TOKEN_META_IDENTIFIER },
	{ TOKEN_CHARACTER_SEQUENCE,         "CHARACTER_SEQUENCE",         TOKEN_META_IDENTIFIER },
	{ TOKEN_BACKSLASH,                  "BACKSLASH",                  TOKEN_META_CHARACTER },
	{ TOKEN_PARENTHESIS_LEFT,           "PARENTHESIS LEFT",           TOKEN_META_CHARACTER },
	{ TOKEN_PARENTHESIS_RIGHT,          "PARENTHESIS RIGHT",          TOKEN_META_CHARACTER },
	{ TOKEN_BRACKET_LEFT,               "BRACKET LEFT",               TOKEN_META_CHARACTER },
	{ TOKEN_BRACKET_RIGHT,              "BRACKET RIGHT",              TOKEN_META_CHARACTER },
	{ TOKEN_BRACE_LEFT,                 "BRACE LEFT",                 TOKEN_META_CHARACTER },
	{ TOKEN_BRACE_RIGHT,                "BRACE RIGHT",                TOKEN_META_CHARACTER },
	{ TOKEN_CLAMP_LEFT,                 "CLAMP LEFT",                 TOKEN_META_CHARACTER },
	{ TOKEN_CLAMP_RIGHT,                "CLAMP RIGHT",                TOKEN_META_CHARACTER },
	{ TOKEN_NOT,                        "NOT",                        TOKEN_META_CHARACTER },
	{ TOKEN_NEG,                        "NEG",                        TOKEN_META_CHARACTER },
	{ TOKEN_MINUS,                      "MINUS",                      TOKEN_META_CHARACTER },
	{ TOKEN_PLUS,                       "PLUS",                       TOKEN_META_CHARACTER },
	{ TOKEN_STAR,                       "STAR",                       TOKEN_META_CHARACTER },
	{ TOKEN_AND,                        "AND",                        TOKEN_META_CHARACTER },
	{ TOKEN_ASSIGN,                     "ASSIGN",                     TOKEN_META_CHARACTER },
	{ TOKEN_SEMICOLON,                  "SEMICOLON",                  TOKEN_META_CHARACTER },
	{ TOKEN_COLON,                      "COLON",                      TOKEN_META_CHARACTER },
	{ TOKEN_COMMA,                      "COMMA",                      TOKEN_META_CHARACTER },
	{ TOKEN_POINT,                      "POINT",                      TOKEN_META_CHARACTER },
	{ TOKEN_XOR,                        "XOR",                        TOKEN_META_CHARACTER },
	{ TOKEN_OR,                         "OR",                         TOKEN_META_CHARACTER },
	{ TOKEN_QUOTE,                      "QUOTE",                      TOKEN_META_CHARACTER },
	{ TOKEN_SLASH,                      "SLASH",                      TOKEN_META_CHARACTER },
	{ TOKEN_MOD,                        "MOD",                        TOKEN_META_CHARACTER },
	{ TOKEN_QUESTION,                   "QUESTION",                   TOKEN_META_CHARACTER },
	// "Tasking 6.0 r4" proprietary extension (start)
	{ TOKEN__BIT,                       "_BIT",                       TOKEN_META_IDENTIFIER },
	{ TOKEN__BITA,                      "_BITA",                      TOKEN_META_IDENTIFIER },
	{ TOKEN__BITWORD,                   "_BITWORD",                   TOKEN_META_IDENTIFIER },
	{ TOKEN__SFRBIT,                    "_SFRBIT",                    TOKEN_META_IDENTIFIER },
	{ TOKEN__ESFRBIT,                   "_ESFRBIT",                   TOKEN_META_IDENTIFIER },
	{ TOKEN__SFR,                       "_SFR",                       TOKEN_META_IDENTIFIER },
	{ TOKEN__ESFR,                      "_ESFR",                      TOKEN_META_IDENTIFIER },
	{ TOKEN__AT,                        "_AT",                        TOKEN_META_IDENTIFIER },
	{ TOKEN__ATBIT,                     "_ATBIT",                     TOKEN_META_IDENTIFIER },
	{ TOKEN__INLINE,                    "_INLINE",                    TOKEN_META_IDENTIFIER },
	{ TOKEN__USM,                       "_USM",                       TOKEN_META_IDENTIFIER },
	{ TOKEN__NOUSM,                     "_NOUSM",                     TOKEN_META_IDENTIFIER },
	{ TOKEN__FAR,                       "_FAR",                       TOKEN_META_IDENTIFIER },
	{ TOKEN__HUGE,                      "_HUGE",                      TOKEN_META_IDENTIFIER },
	{ TOKEN__SHUGE,                     "_SHUGE",                     TOKEN_META_IDENTIFIER },
	{ TOKEN__NEAR,                      "_NEAR",                      TOKEN_META_IDENTIFIER },
	{ TOKEN__SYSTEM,                    "_SYSTEM",                    TOKEN_META_IDENTIFIER },
	{ TOKEN__IRAM,                      "_IRAM",                      TOKEN_META_IDENTIFIER },
	{ TOKEN__GETBIT,                    "_GETBIT",                    TOKEN_META_IDENTIFIER },
	{ TOKEN__PUTBIT,                    "_PUTBIT",                    TOKEN_META_IDENTIFIER },
	{ TOKEN__INTERRUPT,                 "_INTERRUPT",                 TOKEN_META_IDENTIFIER },
	{ TOKEN__USING,                     "_USING",                     TOKEN_META_IDENTIFIER },
	{ TOKEN_INTERRUPT,                  "INTERRUPT",                  TOKEN_META_IDENTIFIER },
	{ TOKEN_USING,                      "USING",                      TOKEN_META_IDENTIFIER },
	{ TOKEN__ROL,                       "_ROL",                       TOKEN_META_IDENTIFIER },
	{ TOKEN__ROR,                       "_ROR",                       TOKEN_META_IDENTIFIER },
	{ TOKEN__TESTCLEAR,                 "_TESTCLEAR",                 TOKEN_META_IDENTIFIER },
	{ TOKEN__TESTSET,                   "_TESTSET",                   TOKEN_META_IDENTIFIER },
	{ TOKEN__BFLD,                      "_BFLD",                      TOKEN_META_IDENTIFIER },
	{ TOKEN__INT166,                    "_INT166",                    TOKEN_META_IDENTIFIER },
	{ TOKEN__IDLE,                      "_IDLE",                      TOKEN_META_IDENTIFIER },
	{ TOKEN__NOP,                       "_NOP",                       TOKEN_META_IDENTIFIER },
	{ TOKEN__PRIOR,                     "_PRIOR",                     TOKEN_META_IDENTIFIER },
	{ TOKEN__PWRDN,                     "_PWRDN",                     TOKEN_META_IDENTIFIER },
	{ TOKEN__SRVWDT,                    "_SRVWDT",                    TOKEN_META_IDENTIFIER },
	{ TOKEN__DISWDT,                    "_DISWDT",                    TOKEN_META_IDENTIFIER },
	{ TOKEN__EINIT,                     "_EINIT",                     TOKEN_META_IDENTIFIER },
	{ TOKEN__ATOMIC,                    "_ATOMIC",                    TOKEN_META_IDENTIFIER },
	{ TOKEN__MUL32,                     "_MUL32",                     TOKEN_META_IDENTIFIER },
	{ TOKEN__MULU32,                    "_MULU32",                    TOKEN_META_IDENTIFIER },
	{ TOKEN__DIV32,                     "_DIV32",                     TOKEN_META_IDENTIFIER },
	{ TOKEN__DIVU32,                    "_DIVU32",                    TOKEN_META_IDENTIFIER },
	{ TOKEN__MOD32,                     "_MOD32",                     TOKEN_META_IDENTIFIER },
	{ TOKEN__MODU32,                    "_MODU32",                    TOKEN_META_IDENTIFIER },
	{ TOKEN__PAG,                       "_PAG",                       TOKEN_META_IDENTIFIER },
	{ TOKEN__POF,                       "_POF",                       TOKEN_META_IDENTIFIER },
	{ TOKEN__SEG,                       "_SEG",                       TOKEN_META_IDENTIFIER },
	{ TOKEN__SOF,                       "_SOF",                       TOKEN_META_IDENTIFIER },
	{ TOKEN__MKFP,                      "_MKFP",                      TOKEN_META_IDENTIFIER },
	{ TOKEN__MKHP,                      "_MKHP",                      TOKEN_META_IDENTIFIER },
	{ TOKEN__MKSP,                      "_MKSP",                      TOKEN_META_IDENTIFIER },
	{ TOKEN_BANK,                       "BANK",                       TOKEN_META_IDENTIFIER },
	{ TOKEN_SFR,                        "SFR",                        TOKEN_META_IDENTIFIER },
	{ TOKEN_ESFR,                       "ESFR",                       TOKEN_META_IDENTIFIER },
	{ TOKEN_SFRBIT,                     "SFRBIT",                     TOKEN_META_IDENTIFIER },
	{ TOKEN_ESFRBIT,                    "ESFRBIT",                    TOKEN_META_IDENTIFIER },
	{ TOKEN_TOK_NEAR,                   "TOK_NEAR",                   TOKEN_META_IDENTIFIER },
	{ TOKEN_TOK_FAR,                    "TOK_FAR",                    TOKEN_META_IDENTIFIER },
	{ TOKEN_TOK_HUGE,                   "TOK_HUGE",                   TOKEN_META_IDENTIFIER },
	{ TOKEN_TOK_SHUGE,                  "TOK_SHUGE",                  TOKEN_META_IDENTIFIER },
	{ TOKEN_BIT,                        "BIT",                        TOKEN_META_IDENTIFIER },
	// "Tasking 6.0 r4" proprietary extension (end)
};

Parser::~Parser()
{}

std::string Parser::tok_name(const Node * node)
{
	return (!node) ? std::string("") : tok_name(node->get_type());
}

int Parser::tok_meta_type(const Node * node)
{
	return (!node) ? TOKEN_META_UNKNOWN : tok_meta_type(node->get_type());
}

std::string Parser::tok_name(int type)
{
	for (size_t i = 0; i < sizeof(token_info_map)/sizeof(token_info_t); ++i)
		if (type == token_info_map[i].type) return std::string(token_info_map[i].name);
	return std::string("UNKNOWN");
}

int Parser::tok_meta_type(int type)
{
	for (size_t i = 0; i < sizeof(token_info_map)/sizeof(token_info_t); ++i)
		if (type == token_info_map[i].type) return token_info_map[i].meta_type;
	return TOKEN_META_UNKNOWN;
}

void Parser::check_for_type_name(Node * spec, Node * decl)
{
	if (spec->sub_count() <= 0) return;
	// FIXME: silly part: extract type name (IDENTIFIER)
	if ((*spec)[0]->is_type(TOKEN_TYPEDEF))
	{
		if (decl->is_type(TOKEN_IDENTIFIER))
		{
			add_type(decl->identifier());
		}
		else if (decl->is_type(TOKEN_DECLARATOR))
		{
			const Node * node = 0;
			for (size_t i = 0; i < decl->size(); ++i)
			{
				node = (*decl)[i];
				if (node->is_type(TOKEN_DIRECT_DECLARATOR))
				{
					if ((*node)[0]->is_type(TOKEN_IDENTIFIER))
					{
						add_type((*(*decl)[i])[0]->identifier());
					}
					break;
				}
				else if (node->is_type(TOKEN_IDENTIFIER))
				{
					add_type(node->identifier());
					break;
				}
			}
		}
		else if (decl->is_type(TOKEN_DIRECT_DECLARATOR))
		{
			const Node * node = 0;
			for (size_t i = 0; i < decl->size(); ++i)
			{
				node = (*decl)[i];
				if (node->is_type(TOKEN_IDENTIFIER))
				{
					add_type(node->identifier());
					break;
				}
				else if (node->is_type(TOKEN_DIRECT_DECLARATOR))
				{
					check_direct_declarator(node);
					break;
				}
			}
		}
	}
}

void Parser::check_direct_declarator(const Node * decl)
{
	const Node * n = 0;
	for (size_t i = 0; i < decl->size(); ++i)
	{
		n = (*decl)[i];
		if (n->is_type(TOKEN_DECLARATOR))
		{
			const Node * m = 0;
			for (size_t j = 0; j < n->size(); ++j)
			{
				m = (*n)[j];
				if (m->is_type(TOKEN_IDENTIFIER))
				{
					add_type(m->identifier());
					return;
				}
			}
		}
	}
}
