#include "TreeReport.h"
#include "Parser.h"
#include <iomanip>
using namespace std;

TreeReport::~TreeReport()
{}

std::string TreeReport::name() const
{
	return string("Tree Report");
}

int TreeReport::print(const STVec & trees, std::ostream & os)
{
	for (STVec::const_iterator tree = trees.begin(); tree != trees.end(); ++tree)
	{
		for (SyntaxTree::const_iterator it = (*tree)->begin(); it != (*tree)->end(); ++it)
		{
			print_tree(os, *it, 0);
		}
	}
	return 0;
}

void TreeReport::print_tree(std::ostream & os, const Node * node, int level) const
{
	os << "[" << setw(60) << node->get_module() << "] ";
	os << "[" << setw(25) << Parser::tok_name(node) << "] ";
	for (int i = 0; i < level; ++i)
	{
		if (i < level-1) os << "---"; else os << "--|";
	}
	switch (Parser::tok_meta_type(node))
	{
		case Parser::TOKEN_META_PSEUDO:
			break;
		case Parser::TOKEN_META_IDENTIFIER:
			os << node->identifier();
			break;
		case Parser::TOKEN_META_CHARACTER:
			os << node->character();
			break;
		default:
			os << "<UNKNOWN>";
			break;
	}
	os << endl;
	for (Node::const_iterator i = node->begin(); i != node->end(); ++i) print_tree(os, *i, level+1);
}
